﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Collections;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.MainForm.WindowsForms;

namespace HeuristicLab.Analysis.Views {
  [View("Aggregated Histogram History View")]
  [Content(typeof(HistogramHistory), IsDefaultView = false)]
  public partial class AggregatedHistogramHistoryView : ItemView {

    public new HistogramHistory Content {
      get { return (HistogramHistory)base.Content; }
      set { base.Content = value; }
    }

    public AggregatedHistogramHistoryView() {
      InitializeComponent();
    }

    protected override void DeregisterContentEvents() {
      Content.ItemsAdded -= new CollectionItemsChangedEventHandler<Histogram>(Content_ItemsAdded);
      Content.ItemsRemoved -= new CollectionItemsChangedEventHandler<Histogram>(Content_ItemsRemoved);
      Content.CollectionReset -= new CollectionItemsChangedEventHandler<Histogram>(Content_CollectionReset);
      foreach (Histogram h in Content) {
        DeregisterHistogramEvents(h);
      }
      base.DeregisterContentEvents();
    }

    private void DeregisterHistogramEvents(Histogram histogram) {
      histogram.Values.ItemsAdded -= new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsAdded);
      histogram.Values.ItemsRemoved -= new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsRemoved);
      histogram.Values.CollectionReset -= new CollectionItemsChangedEventHandler<double>(Content_Values_CollectionReset);
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      Content.ItemsAdded += new CollectionItemsChangedEventHandler<Histogram>(Content_ItemsAdded);
      Content.ItemsRemoved += new CollectionItemsChangedEventHandler<Histogram>(Content_ItemsRemoved);
      Content.CollectionReset += new CollectionItemsChangedEventHandler<Histogram>(Content_CollectionReset);
      foreach (Histogram h in Content) {
        RegisterHistogramEvents(h);
      }
    }

    protected virtual void RegisterHistogramEvents(Histogram histogram) {
      histogram.Values.ItemsAdded += new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsAdded);
      histogram.Values.ItemsRemoved += new CollectionItemsChangedEventHandler<double>(Content_Values_ItemsRemoved);
      histogram.Values.CollectionReset += new CollectionItemsChangedEventHandler<double>(Content_Values_CollectionReset);
    }

    #region Event Handlers (Content)
    private void Content_Values_ItemsAdded(object sender, CollectionItemsChangedEventArgs<double> e) {
      histogramControl.AddPoints(e.Items);
    }
    private void Content_Values_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<double> e) {
      InitializeHistogram();
    }
    private void Content_Values_CollectionReset(object sender, CollectionItemsChangedEventArgs<double> e) {
      InitializeHistogram();
    }
    private void Content_ItemsAdded(object sender, CollectionItemsChangedEventArgs<Histogram> e) {
      List<double> added = new List<double>();
      foreach (Histogram h in e.Items) {
        RegisterHistogramEvents(h);
        added.AddRange(h.Values);
      }
      histogramControl.AddPoints(added);
    }
    private void Content_ItemsRemoved(object sender, CollectionItemsChangedEventArgs<Histogram> e) {
      foreach (Histogram h in e.Items) {
        DeregisterHistogramEvents(h);
      }
      InitializeHistogram();
    }
    private void Content_CollectionReset(object sender, CollectionItemsChangedEventArgs<Histogram> e) {
      foreach (Histogram h in e.OldItems) {
        DeregisterHistogramEvents(h);
      }
      InitializeHistogram();
    }
    #endregion

    protected override void OnContentChanged() {
      base.OnContentChanged();
      if (Content == null) {
        histogramControl.ClearPoints();
      } else {
        InitializeHistogram();
      }
    }

    protected override void SetEnabledStateOfControls() {
      base.SetEnabledStateOfControls();
      histogramControl.Enabled = Content != null;
    }

    private void InitializeHistogram() {
      if (Content == null) return;
      histogramControl.ClearPoints();
      var points = Content.SelectMany(x => x.Values);
      histogramControl.AddPoints(points);
    }

    #region Event Handlers (child controls)
    #endregion
  }
}
